package org.stepik.android.view.injection.serialization

import com.google.gson.Gson
import com.google.gson.GsonBuilder
import dagger.Module
import dagger.Provides
import org.stepic.droid.di.AppSingleton
import org.stepic.droid.jsonHelpers.adapters.CodeOptionsAdapterFactory
import org.stepic.droid.jsonHelpers.adapters.UTCDateAdapter
import org.stepic.droid.jsonHelpers.deserializers.DatasetDeserializer
import org.stepic.droid.jsonHelpers.deserializers.FeedbackDeserializer
import org.stepic.droid.jsonHelpers.deserializers.ReplyDeserializer
import org.stepic.droid.jsonHelpers.serializers.ReplySerializer
import org.stepik.android.domain.catalog.model.CatalogBlock
import org.stepik.android.model.ReplyWrapper
import org.stepik.android.model.attempts.DatasetWrapper
import org.stepik.android.model.feedback.Feedback
import org.stepik.android.remote.catalog.deserializers.CatalogBlockDeserializer
import retrofit2.Converter
import retrofit2.converter.gson.GsonConverterFactory
import java.util.Date

@Module
abstract class SerializationModule {
    @Module
    companion object {
        @Provides
        @AppSingleton
        @JvmStatic
        internal fun provideGsonConverterFactory(gsonInstance: Gson): Converter.Factory =
            GsonConverterFactory.create(gsonInstance)

        @Provides
        @AppSingleton
        @JvmStatic
        internal fun provideGsonInstance(): Gson =
            GsonBuilder()
                .enableComplexMapKeySerialization()
                .registerTypeAdapterFactory(CodeOptionsAdapterFactory())
                .registerTypeAdapter(DatasetWrapper::class.java, DatasetDeserializer())
                .registerTypeAdapter(ReplyWrapper::class.java, ReplyDeserializer())
                .registerTypeAdapter(ReplyWrapper::class.java, ReplySerializer())
                .registerTypeAdapter(Date::class.java, UTCDateAdapter())
                .registerTypeAdapter(Feedback::class.java, FeedbackDeserializer())
                .registerTypeAdapter(CatalogBlock::class.java, CatalogBlockDeserializer())
                .create()
    }
}